@@ -332,28 +332,160 @@ function loadRPCTable(sortKey, sort) {
 	});
 }
 
-function clearMFAQRCode() {
-	document.getElementById('mfa-instructions').innerHTML = "";
+function clearUserInstructions() {
+	document.getElementById('user-instructions').innerHTML = "";
 }
 
 
 function updateMultiFactorAuth(userId, type) {
-	clearMFAQRCode();
+	clearUserInstructions();
 	let req = ajaxRequest("POST", "/admin/update-multi-factor-auth", { "user_id": userId, "type": type });
 	req.then((result) => {
 		if (result) {
-			let html = `<pre id="mfa-instructions" style="line-height: 1.0;">${result}</pre>`;
+			let html = `<pre id="user-instructions" style="line-height: 1.0;">${result}</pre>`;
 			html += "<p>Your multi-factor secret has been changed.<br>";
 			html += "Please use an app like Google Authenticator and scan the QR code displayed.<br>"
 			html += "The app will then generate a new one-time password every 30 seconds.<br>"
-			html += "Without this password you will not be able to log in to the opsi server anymore.</p>"
-			html += '<button onClick="clearMFAQRCode();">All done, hide instructions and QR code.</button>';
-			document.getElementById("mfa-instructions").innerHTML = html;
+			html += "Without this password you will not be able to log in to the OPSI server anymore.</p>"
+			html += '<button onClick="clearUserInstructions();">All done, hide instructions and QR code.</button>';
+			document.getElementById("user-instructions").innerHTML = html;
 		}
 		loadUserTable();
 	});
 }
 
+function showPasswordInstructions(userId, errorMessage = null) {
+	let html = `<h4>Change password</h4>`;
+	if (errorMessage) {
+		html += `<p style="color:red;">${errorMessage}</p>`;
+	}
+	html += `<form action="javascript:void(0);">`;
+	html += `<label for="username" style="width: 150px; display: inline-block;">Username:</label>`;
+	html += `<input type="text" id="username" autocomplete="username" value="${userId}" /><br/>`;
+	html += `<label for="new-password" style="width: 150px; display: inline-block;">New password:</label>`;
+	html += `<input type="password" id="new-password" autocomplete="new-password" /><br/>`;
+	html += `<label for="confirm-password" style="width: 150px; display: inline-block;">Confirm password:</label>`;
+	html += `<input type="password" id="confirm-password" autocomplete="new-password" /><br/>`;
+	html += `<button onclick="changeInternalUserPassword('${userId}', document.getElementById('new-password').value, document.getElementById('confirm-password').value);">Change password</button>`;
+	html += `</form>`;
+	document.getElementById("user-instructions").innerHTML = html;
+}
+
+function changeInternalUserPassword(userId, password = null, confirmPassword = null) {
+	if (!password || password !== confirmPassword) {
+		let errorMessage = null;
+		if (password !== confirmPassword) {
+			errorMessage = "The provided passwords do not match. Please try again.";
+		}
+		showPasswordInstructions(userId, errorMessage);
+	}
+	else {
+		let req = ajaxRequest("POST", "/admin/set-internal-user-password", { "user_id": userId, "password": password });
+		req.then((result) => {
+			showNotifcation("New password successfully set", "user-edit", "success", 3);
+			clearUserInstructions();
+			loadUserTable();
+		}, (error) => {
+			let errorMessage = `Failed to set user password: ${error.message || error.detail || JSON.stringify(error)}`;
+			showPasswordInstructions(userId, errorMessage);
+			showNotifcation(errorMessage, "user-edit", "error", 10);
+			loadUserTable();
+		});
+	}
+}
+
+function updateUserTokenAuth(userId, enable) {
+	let req = ajaxRequest("POST", "/admin/update-user-token-auth", { "user_id": userId, "enable": enable });
+	req.then((result) => {
+		showNotifcation(`Token authentication ${enable ? "enabled" : "disabled"} successfully`, "user-edit", "success", 3);
+		if (enable) {
+			let html = "<p>Token authentication is now enabled.<br>";
+			html += "Please store the following token securely as it will not be shown again:<br><br>";
+			html += `<strong>${result}</strong></p>`;
+			html += '<button onClick="clearUserInstructions();">Done, hide token</button>';
+			document.getElementById("user-instructions").innerHTML = html;
+		}
+		else {
+			clearUserInstructions();
+		}
+		loadUserTable();
+	}, (error) => {
+		let errorMessage = `Failed to update token authentication: ${error.message || error.detail || JSON.stringify(error)}`;
+		showNotifcation(errorMessage, "user-edit", "error", 10);
+		loadUserTable();
+	});
+}
+
+
+function showNewUserInstructions(userId, admin, readonly, errorMessage = null) {
+	let html = `<h4>Create new user</h4>`;
+	if (errorMessage) {
+		html += `<p style="color:red;">${errorMessage}</p>`;
+	}
+	html += `<form action="javascript:void(0);">`;
+	html += `<label for="username" style="width: 150px; display: inline-block;">Username:</label>`;
+	html += `<input type="text" id="username" autocomplete="username" value="${userId || ''}" /><br/>`;
+	html += `<label for="new-password" style="width: 150px; display: inline-block;">Password:</label>`;
+	html += `<input type="password" id="new-password" autocomplete="new-password" /><br/>`;
+	html += `<label for="confirm-password" style="width: 150px; display: inline-block;">Confirm password:</label>`;
+	html += `<input type="password" id="confirm-password" autocomplete="new-password" /><br/>`;
+	html += `<input type="checkbox" id="admin" ${admin ? "checked" : ""} onchange="document.getElementById('readonly').checked = !this.checked;" /> Admin<br/>`;
+	html += `<input type="checkbox" id="readonly" ${readonly ? "checked" : ""} onchange="document.getElementById('admin').checked = !this.checked;" /> Readonly<br/>`;
+	html += `<button onclick="createUser(document.getElementById('username').value, document.getElementById('new-password').value, document.getElementById('confirm-password').value, document.getElementById('admin').checked, document.getElementById('readonly').checked);">Create user</button>`;
+	html += `</form>`;
+	document.getElementById("user-instructions").innerHTML = html;
+}
+
+function createUser(userId = null, password = null, confirmPassword = null, admin = true, readonly = false) {
+	if (!userId || !password || password !== confirmPassword) {
+		let errorMessage = null;
+		if (password !== confirmPassword) {
+			errorMessage = "The provided passwords do not match. Please try again.";
+		}
+		showNewUserInstructions(userId, admin, readonly, errorMessage);
+	}
+	else {
+		let req = ajaxRequest("POST", "/admin/create-user", { "user_id": userId, "password": password, "admin": admin, "readonly": readonly });
+		req.then((result) => {
+			showNotifcation("New user successfully created", "user-edit", "success", 3);
+			clearUserInstructions();
+			loadUserTable();
+		}, (error) => {
+			let errorMessage = `Failed to create user: ${error.message || error.detail || JSON.stringify(error)}`;
+			showNewUserInstructions(userId, admin, readonly, errorMessage);
+			showNotifcation(errorMessage, "user-edit", "error", 10);
+			loadUserTable();
+		});
+	}
+}
+
+
+function deleteUser(userId, confirmed = false) {
+	if (!confirmed) {
+		let html = `<h4>Delete user ${userId}</h4>`;
+		html += `<p>Are you sure you want to delete the user <strong>${userId}</strong>?`;
+		html += `<br />This action cannot be undone.</p>`;
+		html += `<form action="javascript:void(0);">`;
+		html += `<button onclick="clearUserInstructions()">Cancel</button> `;
+		html += `<button onclick="deleteUser('${userId}', true)">Delete user</button>`;
+		html += `</form>`;
+		document.getElementById("user-instructions").innerHTML = html;
+	}
+	else {
+		let req = ajaxRequest("POST", "/admin/delete-user", { "user_id": userId });
+		req.then((result) => {
+			showNotifcation("User successfully deleted", "user-edit", "success", 3);
+			clearUserInstructions();
+			loadUserTable();
+		}, (error) => {
+			let errorMessage = `Failed to delete user: ${error.message || error.detail || JSON.stringify(error)}`;
+			showNotifcation(errorMessage, "user-edit", "error", 10);
+			clearUserInstructions();
+			loadUserTable();
+		});
+	}
+}
+
 
 function printUserTable(data, htmlId) {
 	if (data.length == 0) {
@@ -363,38 +495,59 @@ function printUserTable(data, htmlId) {
 		htmlStr = "<table class=\"user-table\" id=\"user-table\">" +
 			"<tr>" +
 			"<th class='user-th'>User-ID</th>" +
+			"<th class='user-th'>Groups</th>" +
 			"<th class='user-th'>Last login</th>" +
-			"<th class='user-th'>Messagebus</th>";
+			"<th class='user-th'>Internal Authentication</th>" +
+			"<th class='user-th'>Token Authentication</th>";
 		if (multiFactorAuth == "totp_optional" || multiFactorAuth == "totp_mandatory") {
 			htmlStr += "<th class='user-th'>MFA state</th>";
-			htmlStr += "<th class='user-th'>Activate Time-based one-time password</th>";
-			if (multiFactorAuth == "totp_optional") {
-				htmlStr += "<th class='user-th'>Deactivate Multi-factor auth</th>";
-			}
 		}
+		htmlStr += "<th class='user-th'>Messagebus</th>";
+		htmlStr += "<th class='user-th'>Actions</th>";
 		htmlStr += "</tr>";
 		data.forEach(user => {
-			let connected = messagebusConnectedUsers.includes(user.id);
-			let cls = "user-" + (connected ? "connected" : "not-connected");
+			let actions = [];
 			htmlStr += "<tr>" +
 				`<td class="user-td">${user.id}</td>` +
-				`<td class="user-td">${formateDate(new Date(user.lastLogin))}</td>` +
-				`<td id="user-messagebus-state-${user.id}" data-user-id="${user.id}" class="user-td ${cls}">` +
-				`${connected ? 'connected' : 'not connected'}</td >`;
+				`<td class="user-td" style="max-width: 150px">${user.groups.join(", ")}</td>` +
+				`<td class="user-td">${user.lastLogin ? formateDate(new Date(user.lastLogin)) : "never"}</td>` +
+				`<td class="user-td">${user.internal_auth ? "yes" : "no"}</td>` +
+				`<td class="user-td">${user.token_auth ? "yes" : "no"}</td>`;
+			if (databaseAuth) {
+				if (user.internal_auth) {
+					actions.push(`<input type="button" onclick="changeInternalUserPassword('${user.id}')" value="Change password"></input>`);
+				}
+				if (user.token_auth) {
+					actions.push(`<input type="button" onclick="updateUserTokenAuth('${user.id}', false)" value="Remove Authentication Token"></input>`);
+				}
+				actions.push(`<input type="button" onclick="updateUserTokenAuth('${user.id}', true)" value="New Authentication Token"></input>`);
+			}
 			if (multiFactorAuth == "totp_optional" || multiFactorAuth == "totp_mandatory") {
-				cls = "mfa-" + (user.mfaState == "inactive" ? "inactive" : "active");
+				let cls = "mfa-" + (user.mfaState == "inactive" ? "inactive" : "active");
 				if (multiFactorAuth == "totp_mandatory" && user.mfaState == "inactive") {
 					cls += "-warn";
 				}
 				htmlStr += `<td class="user-td ${cls}">${user.mfaState}</td>`;
-				htmlStr += `<td class="user-td"><input type="button" onclick="updateMultiFactorAuth('${user.id}', 'totp')" value="Generate new secret and activate TOTP"</td>`;
-				if (multiFactorAuth == "totp_optional") {
-					htmlStr += `<td class="user-td"><input type="button" onclick="updateMultiFactorAuth('${user.id}', 'inactive')" value="Deactivate MFA"</td>`;
+				actions.push(`<input type="button" onclick="updateMultiFactorAuth('${user.id}', 'totp')" value="Generate new secret and activate TOTP"></input>`);
+				if (multiFactorAuth == "totp_optional" && user.mfaState != "inactive") {
+					actions.push(`<input type="button" onclick="updateMultiFactorAuth('${user.id}', 'inactive')" value="Deactivate MFA"></input>`);
 				}
 			}
+			if (databaseAuth) {
+				actions.push(`<input type="button" onclick="deleteUser('${user.id}')" value="Delete user"></input>`);
+			}
+			let connected = messagebusConnectedUsers.includes(user.id);
+			let cls = "user-" + (connected ? "connected" : "not-connected");
+			htmlStr += `<td class="user-td ${cls}"id="user-messagebus-state-${user.id}" data-user-id="${user.id}">` +
+				`${connected ? 'connected' : 'not connected'}</td >`;
+			htmlStr += `<td class="user-td">${actions.join(" ")}</td>`;
+			htmlStr += "</tr>";
 		});
 		htmlStr += "</table>";
 	}
+	if (databaseAuth) {
+		htmlStr += "<button onclick='createUser();'>Create new user</button>";
+	}
 	div = document.getElementById(htmlId);
 	div.innerHTML = htmlStr;
 	return htmlStr;
